"""
The contacts module provides functionality for working with contacts in your
ActiveCampaign account.
"""

from activecampaign import http


class Contacts(object):

    def __init__(self, client):
        self.client = client

    def view(self, id):
        """
        Return an individual contact

        Args:
            id: an integer ID or email for a contact

        Returns:
            Dictionary of contact information
        """
        return self.client.request(http.GET, "contact_view", [('id', id)])

    def sync(self, **kwargs):
        if 'email' not in kwargs:
            raise ValueError("You must include the email field")
        return self.client.request(http.POST, "contact_sync", kwargs.items())

    def tag_add(self, *tags, **kwargs):
        """
        Adds one or more tags to a contact

        The arguments `email` and `id` are mutually exclusive. If both are provided
        then only `id` will be used.

        Args:
            email: the email address of the ActiveCampaign contact
            id: the id of the ActiveCampaign contact
            *tags: list or iterable of string tag names

        Returns:

        """
        id = kwargs.pop('id', None)
        email = kwargs.pop('email', None)

        data = [('tags', tag) for tag in tags]
        if email is None and id is None:
            raise TypeError("Must provide `email` or `id` keyword argument")

        if id is not None:
            data.append(['id', id])
        else:
            data.append(['email', email])

        return self.client.request(http.POST, "contact_tag_add", data)

    def tag_remove(self, *tags, **kwargs):
        """
        Removes one or more tags from a contact

        The arguments `email` and `id` are mutually exclusive. If both are provided
        then only `id` will be used.

        Args:
            email: the email address of the ActiveCampaign contact
            id: the id of the ActiveCampaign contact
            *tags: list or iterable of string tag names

        Returns:

        """
        id = kwargs.pop('id', None)
        email = kwargs.pop('email', None)

        data = [('tags', tag) for tag in tags]
        if email is None and id is None:
            raise TypeError("Must provide `email` or `id` keyword argument")

        if id is not None:
            data.append(['id', id])
        else:
            data.append(['email', email])

        return self.client.request(http.POST, "contact_tag_remove", data)



"""
The client module is responsible for abstracting away connecting to, making
requests to, and serializing responses from the ActiveCampaign API.
"""

import requests

from .contacts import Contacts


class ActiveCampaignClient(object):

    def __init__(self, host, key):
        """
        Requests are issued against an account specific domain
        """
        self.host = host
        self.key = key
        self.contacts = Contacts(self)

    def request(self, method, action, data=None):
        """

        Args:
            method: the HTTP method to use as an uppercase string
            action: the name of the URL parameter defining the API action
            data: a list of tuples for the data. Used in lieu of a dictionary to allow
                    for key duplication

        Returns:

        """
        url = "{0}/admin/api.php".format(self.host)
        params = [
            ('api_action', action),
            ('api_key', self.key),
            ('api_output', 'json'),
        ]
        if method in ('GET', 'DELETE'):
            if data is not None:
                params += data
            data = []
        else:
            data = data

        response = requests.request(method, url, params=params, data=data)
        return response
